window.Gleam.initSearch([{"doc":"sqlight","title":"sqlight","content":"# sqlight\n\n[![Package Version](https://img.shields.io/hexpm/v/sqlight)](https://hex.pm/packages/sqlight)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/sqlight/)\n\nUse [SQLite](https://www.sqlite.org/index.html) from Gleam!\n\nWorks on Erlang or JavaScript running on Deno.\n\n```sh\ngleam add sqlight\n```\n\n```gleam\nimport gleam/dynamic/decode\nimport sqlight\n\npub fn main() {\n  use conn &lt;- sqlight.with_connection(&quot;:memory:&quot;)\n  let cat_decoder = {\n    use name &lt;- decode.field(0, decode.string)\n    use age &lt;- decode.field(1, decode.int)\n    decode.success(#(name, age))\n  }\n\n  let sql = &quot;\n  create table cats (name text, age int);\n\n  insert into cats (name, age) values \n  (&#39;Nubi&#39;, 4),\n  (&#39;Biffy&#39;, 10),\n  (&#39;Ginny&#39;, 6);\n  &quot;\n  let assert Ok(Nil) = sqlight.exec(sql, conn)\n\n  let sql = &quot;\n  select name, age from cats\n  where age &lt; ?\n  &quot;\n  let assert Ok([#(&quot;Nubi&quot;, 4), #(&quot;Ginny&quot;, 6)]) =\n    sqlight.query(sql, on: conn, with: [sqlight.int(7)], expecting: cat_decoder)\n}\n```\n\nDocumentation can be found at &lt;https://hexdocs.pm/sqlight&gt;.\n\n## Why SQLite?\n\nSQLite is a implementation of SQL as a library. This means that you don&#39;t run a\nseparate SQL server that your program communicates with, but you embed the SQL\nimplementation directly in your program. SQLite stores its data in a single\nfile. The file format is portable between different machine architectures. It\nsupports atomic transactions and it is possible to access the file by multiple\nprocesses and different programs.\n\nYou can also use in-memory databases with SQLite, which may be useful for testing.\n\n## Implementation\n\nWhen running on Erlang is library wrapper around the excellent Erlang library\n[esqlite](https://hex.pm/packages/esqlite), which in turn is a wrapper around\nthe SQLite C library. It is implemented as a NIF, which means that the SQLite\ndatabase engine is linked to the erlang virtual machine.\n\nWhen running on Deno it is a wrapper around the excellent\n[x/sqlite](https://deno.land/x/sqlite@v3.7.0) library, which in turn is a\nwrapper around the SQLite C library compiled to WASM.\n\n## On using Bool with SQLite\n\nSQLite does not have a native boolean type. Instead, it uses ints, where 0 is\nFalse and 1 is True. Because of this the Gleam stdlib decoder for bools will not\nwork, instead the `sqlight.decode_bool` function should be used as it supports\nboth ints and bools.\n","url":"index.html"},{"doc":"sqlight","title":"Connection","content":"pub type Connection\n\n\nSynonyms:\nsqlight.Connection\nsqlight Connection","url":"sqlight.html#Connection"},{"doc":"sqlight","title":"Error","content":"pub type Error {\n  SqlightError(code: ErrorCode, message: String, offset: Int)\n}\n\nSqlightError(code: ErrorCode, message: String, offset: Int)\n\noffset\n&lt;p&gt;If the most recent error references a specific token in the input SQL,\nthis is the byte offset of the start of that token.\nIf the most recent error does not reference a specific token, this is -1.&lt;/p&gt;\n\nSynonyms:\nsqlight.Error\nsqlight Error","url":"sqlight.html#Error"},{"doc":"sqlight","title":"ErrorCode","content":"pub type ErrorCode {\n  Abort\n  Auth\n  Busy\n  Cantopen\n  Constraint\n  Corrupt\n  Done\n  Empty\n  GenericError\n  Format\n  Full\n  Internal\n  Interrupt\n  Ioerr\n  Locked\n  Mismatch\n  Misuse\n  Nolfs\n  Nomem\n  Notadb\n  Notfound\n  Notice\n  GenericOk\n  Perm\n  Protocol\n  Range\n  Readonly\n  Row\n  Schema\n  Toobig\n  Warning\n  AbortRollback\n  AuthUser\n  BusyRecovery\n  BusySnapshot\n  BusyTimeout\n  CantopenConvpath\n  CantopenDirtywal\n  CantopenFullpath\n  CantopenIsdir\n  CantopenNotempdir\n  CantopenSymlink\n  ConstraintCheck\n  ConstraintCommithook\n  ConstraintDatatype\n  ConstraintForeignkey\n  ConstraintFunction\n  ConstraintNotnull\n  ConstraintPinned\n  ConstraintPrimarykey\n  ConstraintRowid\n  ConstraintTrigger\n  ConstraintUnique\n  ConstraintVtab\n  CorruptIndex\n  CorruptSequence\n  CorruptVtab\n  ErrorMissingCollseq\n  ErrorRetry\n  ErrorSnapshot\n  IoerrAccess\n  IoerrAuth\n  IoerrBeginAtomic\n  IoerrBlocked\n  IoerrCheckreservedlock\n  IoerrClose\n  IoerrCommitAtomic\n  IoerrConvpath\n  IoerrCorruptfs\n  IoerrData\n  IoerrDelete\n  IoerrDeleteNoent\n  IoerrDirClose\n  IoerrDirFsync\n  IoerrFstat\n  IoerrFsync\n  IoerrGettemppath\n  IoerrLock\n  IoerrMmap\n  IoerrNomem\n  IoerrRdlock\n}\n The errors that SQLite can return.\n\n See the SQLite documentation for further details.\n &lt;https://sqlite.org/rescode.html&gt;\n\n When running on JavaScript with Deno only a less-detailed subset of these\n will be used as the underlying library does not expose extended error codes.\n\nAbort\n\n\nAuth\n\n\nBusy\n\n\nCantopen\n\n\nConstraint\n\n\nCorrupt\n\n\nDone\n\n\nEmpty\n\n\nGenericError\n\n\nFormat\n\n\nFull\n\n\nInternal\n\n\nInterrupt\n\n\nIoerr\n\n\nLocked\n\n\nMismatch\n\n\nMisuse\n\n\nNolfs\n\n\nNomem\n\n\nNotadb\n\n\nNotfound\n\n\nNotice\n\n\nGenericOk\n\n\nPerm\n\n\nProtocol\n\n\nRange\n\n\nReadonly\n\n\nRow\n\n\nSchema\n\n\nToobig\n\n\nWarning\n\n\nAbortRollback\n\n\nAuthUser\n\n\nBusyRecovery\n\n\nBusySnapshot\n\n\nBusyTimeout\n\n\nCantopenConvpath\n\n\nCantopenDirtywal\n\n\nCantopenFullpath\n\n\nCantopenIsdir\n\n\nCantopenNotempdir\n\n\nCantopenSymlink\n\n\nConstraintCheck\n\n\nConstraintCommithook\n\n\nConstraintDatatype\n\n\nConstraintForeignkey\n\n\nConstraintFunction\n\n\nConstraintNotnull\n\n\nConstraintPinned\n\n\nConstraintPrimarykey\n\n\nConstraintRowid\n\n\nConstraintTrigger\n\n\nConstraintUnique\n\n\nConstraintVtab\n\n\nCorruptIndex\n\n\nCorruptSequence\n\n\nCorruptVtab\n\n\nErrorMissingCollseq\n\n\nErrorRetry\n\n\nErrorSnapshot\n\n\nIoerrAccess\n\n\nIoerrAuth\n\n\nIoerrBeginAtomic\n\n\nIoerrBlocked\n\n\nIoerrCheckreservedlock\n\n\nIoerrClose\n\n\nIoerrCommitAtomic\n\n\nIoerrConvpath\n\n\nIoerrCorruptfs\n\n\nIoerrData\n\n\nIoerrDelete\n\n\nIoerrDeleteNoent\n\n\nIoerrDirClose\n\n\nIoerrDirFsync\n\n\nIoerrFstat\n\n\nIoerrFsync\n\n\nIoerrGettemppath\n\n\nIoerrLock\n\n\nIoerrMmap\n\n\nIoerrNomem\n\n\nIoerrRdlock\n\n\nSynonyms:\nsqlight.ErrorCode\nsqlight ErrorCode","url":"sqlight.html#ErrorCode"},{"doc":"sqlight","title":"Stats","content":"pub type Stats {\n  Stats(used: Int, highwater: Int)\n}\n\nStats(used: Int, highwater: Int)\n\n\nSynonyms:\nsqlight.Stats\nsqlight Stats","url":"sqlight.html#Stats"},{"doc":"sqlight","title":"Value","content":"pub type Value\n A value that can be sent to SQLite as one of the arguments to a\n parameterised SQL query.\n\nSynonyms:\nsqlight.Value\nsqlight Value","url":"sqlight.html#Value"},{"doc":"sqlight","title":"blob","content":"pub fn blob(value: BitArray) -&gt; Value\n Convert a Gleam `BitString` to an SQLite blob, to be used an argument to a\n query.\n\nSynonyms:\nsqlight.blob\nsqlight blob","url":"sqlight.html#blob"},{"doc":"sqlight","title":"bool","content":"pub fn bool(value: Bool) -&gt; Value\n Convert a Gleam `Bool` to an SQLite int, to be used an argument to a\n query.\n\n SQLite does not have a native boolean type. Instead, it uses ints, where 0\n is False and 1 is True. Because of this the Gleam stdlib decoder for bools\n will not work, instead the `decode_bool` function should be used as it\n supports both ints and bools.\n\nSynonyms:\nsqlight.bool\nsqlight bool","url":"sqlight.html#bool"},{"doc":"sqlight","title":"close","content":"pub fn close(connection: Connection) -&gt; Result(Nil, Error)\n Close a connection to a SQLite database.\n\n Ideally applications should finallise all prepared statements and other open\n resources before closing a connection. See the SQLite documentation for more\n information: &lt;https://www.sqlite.org/c3ref/close.html&gt;.\n\nSynonyms:\nsqlight.close\nsqlight close","url":"sqlight.html#close"},{"doc":"sqlight","title":"decode_bool","content":"pub fn decode_bool() -&gt; Decoder(Bool)\n Decode an SQLite boolean value.\n\n Decodes 0 as `False` and any other integer as `True`.\n\nSynonyms:\nsqlight.decode_bool\nsqlight decode_bool","url":"sqlight.html#decode_bool"},{"doc":"sqlight","title":"error_code_from_int","content":"pub fn error_code_from_int(code: Int) -&gt; ErrorCode\n Convert an error code int to an `Error`.\n\n If the code is not a known error code, `GenericError` is returned.\n\nSynonyms:\nsqlight.error_code_from_int\nsqlight error_code_from_int","url":"sqlight.html#error_code_from_int"},{"doc":"sqlight","title":"error_code_to_int","content":"pub fn error_code_to_int(error: ErrorCode) -&gt; Int\n Convert an `Error` to an error code int.\n\n See the SQLite documentation for the full list of error codes.\n &lt;https://sqlite.org/rescode.html&gt;\n\nSynonyms:\nsqlight.error_code_to_int\nsqlight error_code_to_int","url":"sqlight.html#error_code_to_int"},{"doc":"sqlight","title":"exec","content":"pub fn exec(\n  sql: String,\n  on connection: Connection,\n) -&gt; Result(Nil, Error)\n\nSynonyms:\nsqlight.exec\nsqlight exec","url":"sqlight.html#exec"},{"doc":"sqlight","title":"float","content":"pub fn float(value: Float) -&gt; Value\n Convert a Gleam `Float` to an SQLite float, to be used an argument to a\n query.\n\nSynonyms:\nsqlight.float\nsqlight float","url":"sqlight.html#float"},{"doc":"sqlight","title":"int","content":"pub fn int(value: Int) -&gt; Value\n Convert a Gleam `Int` to an SQLite int, to be used an argument to a\n query.\n\nSynonyms:\nsqlight.int\nsqlight int","url":"sqlight.html#int"},{"doc":"sqlight","title":"null","content":"pub fn null() -&gt; Value\n Construct an SQLite null, to be used an argument to a query.\n\nSynonyms:\nsqlight.null\nsqlight null","url":"sqlight.html#null"},{"doc":"sqlight","title":"nullable","content":"pub fn nullable(\n  inner_type: fn(a) -&gt; Value,\n  value: Option(a),\n) -&gt; Value\n Convert a Gleam `Option` to an SQLite nullable value, to be used an argument\n to a query.\n\nSynonyms:\nsqlight.nullable\nsqlight nullable","url":"sqlight.html#nullable"},{"doc":"sqlight","title":"open","content":"pub fn open(path: String) -&gt; Result(Connection, Error)\n Open a connection to a SQLite database.\n\n URI filenames are supported by SQLite, making it possible to open read-only\n databases, in memory databases, and more. Further information about this can\n be found in the SQLite documentation: &lt;https://sqlite.org/uri.html&gt;.\n\n # Examples\n\n ## Open &quot;data.db&quot; in the current working directory\n\n \n let assert Ok(conn) = open(&quot;file:data.sqlite3&quot;)\n \n \n ## Opens &quot;data.db&quot; in read only mode with a private cache\n \n \n let assert Ok(conn) = open(&quot;file:data.db?mode=ro&amp;cache=private&quot;)\n \n \n Opens a shared memory database named memdb1 with a shared cache. \n \n \n let assert Ok(conn) = open(&quot;file:memdb1?mode=memory&amp;cache=shared&quot;)\n \n\nSynonyms:\nsqlight.open\nsqlight open","url":"sqlight.html#open"},{"doc":"sqlight","title":"query","content":"pub fn query(\n  sql: String,\n  on connection: Connection,\n  with arguments: List(Value),\n  expecting decoder: Decoder(a),\n) -&gt; Result(List(a), Error)\n\nSynonyms:\nsqlight.query\nsqlight query","url":"sqlight.html#query"},{"doc":"sqlight","title":"text","content":"pub fn text(value: String) -&gt; Value\n Convert a Gleam `String` to an SQLite text, to be used an argument to a\n query.\n\nSynonyms:\nsqlight.text\nsqlight text","url":"sqlight.html#text"},{"doc":"sqlight","title":"with_connection","content":"pub fn with_connection(path: String, f: fn(Connection) -&gt; a) -&gt; a\n Open a connection to a SQLite database and execute a function with it, then\n close the connection.\n\n This function works well with a `use` expression to automatically close the\n connection at the end of a block.\n\n # Crashes\n\n This function crashes if the connection cannot be opened or closed.\n\n # Examples\n\n \n use conn &lt;- with_connection(&quot;file:mydb?mode=memory&quot;)\n // Use the connection here...\n \n\nSynonyms:\nsqlight.with_connection\nsqlight with_connection","url":"sqlight.html#with_connection"},{"doc":"sqlight","title":"sqlight","content":"","url":"sqlight.html"}]);